" autocmdgroup.vim
" @Author:      Tom Link (mailto:micathom AT gmail com?subject=[vim])
" @Website:     http://www.vim.org/account/profile.php?user_id=4037
" @License:     GPL (see http://www.gnu.org/licenses/gpl.txt)
" @Revision:    7

augroup TLib
    autocmd!
augroup END


function! tlib#autocmdgroup#Init() "{{{3
endf

